#!/usr/bin/python3.2
# -*- coding: utf-8 -*-

import random

def load_dico(filename):
    """
    Desc:   Return a set of words loaded from a given file.
    Input:  filename (str)
    Output: dico (set)
    """
    f = open(filename)
    return set(f.read().split())

def get_word_from_dico(dico):
    """
    Desc:   Return a randomly selected word from a set.
    Input:  dico (set)
    Output: word (str)
    """
    return random.sample(dico, 1)[0].lower()

def ask_yes_no(message):
    """
    Desc:   Return 'True' if a user's answer to the message received in
                parameter is 'o', and 'False' if it's answer is 'n'. 
            The question is asked until the answer is either 'o' or 'n'.
    Input:  message (str)
    Output: answer (bool)
    """
    answer = input(message+" o/n ").lower()
    while answer != "o" and answer != "n":
        answer = input(message+" o/n ").lower()
    return answer == "o"

def ask_letter(message):
    """
    Desc:   Return the answer of a user to the message received in parameter.
            The question is asked until the answer is one alphabetic letter.
    Input:  message (str)
    Output: answer (str)
    """
    answer = input(message)
    while len(answer) != 1 or not answer.isalpha():
        print("Entrée erronée, veuillez réessayer")
        answer = input(message)
    return answer

def update_guess(word,word_guessed,letter):
    """
    Desc:   Check for occurences of 'letter' in 'word' and replace those 
                letters in 'word_guessed' at the same positions.
            Return the modified 'word_guessed'
    Input:  word (str) , word_guessed (str) , letter (str)
    Output: word_guessed (str)
    """
    word_guessed = list(word_guessed)
    for i in range(len(word)):
        if word[i] == letter:
            word_guessed[i] = letter
    return "".join(word_guessed)

def generate_guess(word):    
    """
    Desc:   Return a string of 'word's length composed only of '_'.
    Input:  word (str)
    Output: word_guessed (str)
    """
    
    return "_" * len(word)

def init_game(dico):
    """
    Desc:   Return the variables of the game (number of trials and word to guess)
    Input:  dico (set)
    Output: trials (int), word, (str)
    """
    trials = 10
    word = get_word_from_dico(dico)
    return trials, word

def show_status(trials, word_guessed):
    """
    Desc:   Show the status of the game (number of trials and word to guess)
    Input:  trials (int), word_guessed (str)
    Output: 
    """
    print("Il vous reste " + str(trials) + " essais")
    print("Le mot est actuellement : ",word_guessed)
    
def hang(dico):    
    """
    Desc:   Execute one game of 'Hanged Man' using the set 'dico'.
    Input:  dico (set)  
    Output:
    """
    trials, word = init_game(dico)
    word_guessed = generate_guess(word)
    show_status(trials, word_guessed)
    while trials != 0 and not check_win(word_guessed, word):
        letter = ask_letter("Indiquez une lettre : ").lower()
        if letter not in word:
            trials -= 1
        else:
            word_guessed = update_guess(word,word_guessed,letter)
        show_status(trials, word_guessed)
    endgame(word_guessed, word)

def check_win(word_guessed, word):    
    """
    Desc:   Check if the word has been guessed.
            Return 'True' if the word has been guessed, 'False' otherwise.
    Input:  win (str, str)
    Output: (bool)
    """
    return word_guessed == word
    
def endgame(word_guessed, word):
    """
    Desc:   Print the end game message.
    Input:  endgame (bool)
    Output: 
    """
    if check_win(word_guessed, word):
        print("Bravo, vous avez gagné !")
    else:
        print("Dommage, vous avez perdu...")
        print("Le mot à deviner était : " + word)

def game():
    """
    Desc:   Start a new game until the user wants to stop playing.
    Input:    
    Output:    
    """
    dico = load_dico("dico.txt")
    hang(dico)
    while(ask_yes_no("Faire une nouvelle partie ?")):
        hang(dico)

if __name__ == "__main__":
    game()
