#! /usr/bin/env python2

from random import random
from pylab import plot, show

def noise(x, margin):
    """
    Adds random noise to the value x. The relative error of the produced value
    is randomly chosen between -margin and +margin.
    """
    error = (random()-.5) * 2 * margin * abs(x)
    return error + x

def draw(points):
    """
    Takes a list of points as argument and draws them on a plot as a cloud of
    points.
    """
    for point in points:
        plot(point[0], point[1], 'ro')
    show()

def random_values(start, end, n):
    """
    Generates n random values between start included and end not included.
    """
    res = []
    for _ in range(n):
        res.append(random() * (end - start) + start)
    return res

def map(f, p):
    """
    Returns a new list the same size as the sequence p, where each element of
    the new list is the result of applying the function f to the corresponding
    element in the list p.
    """
    res = []
    for el in p:
        res.append(f(el))
    return res

def noisy_points(x, func):
    y = map(func, x)
    noisy_y = map(lambda x: noise(x, .1), y)
    return list(zip(x, y))

if __name__ == "__main__":
    points = noisy_points(random_values(-5, 5, 30), lambda x: .1 * x ** 3 + 10 * x - 3)
    #draw(points)
    with open("data_tp_6.txt", "w") as f:
        for (x, y) in sorted(points):
            f.write("{} {}\n".format(x, y))
